package ch.sahits.game.graphic.image;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.ImageCapabilities;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.awt.image.Raster;
import java.awt.image.RenderedImage;
import java.awt.image.SampleModel;
import java.awt.image.TileObserver;
import java.awt.image.WritableRaster;
import java.util.Vector;
/**
 * This is a wrapper class of the {@link BufferedImage} to assign it a name.
 * @author Andi Hotz, (c) Sahits GmbH, 2011
 * Created on Mar 4, 2011
 *
 */
public class NamedBufferedImage {

	private final BufferedImage image;
	private final String name;
	public NamedBufferedImage(BufferedImage image, String name) {
		super();
		if (image==null){
			throw new NullPointerException("The image may not be null");
		}
		if (name==null){
			throw new NullPointerException("The name may not be null");
		}
		this.image = image;
		this.name = name;
	}
	public BufferedImage getImage() {
		return image;
	}
	public String getName() {
		return name;
	}
	public int hashCode() {
		return image.hashCode();
	}
	public boolean equals(Object obj) {
		return image.equals(obj);
	}
	public Image getScaledInstance(int width, int height, int hints) {
		return image.getScaledInstance(width, height, hints);
	}
	public void flush() {
		image.flush();
	}
	public ImageCapabilities getCapabilities(GraphicsConfiguration gc) {
		return image.getCapabilities(gc);
	}
	public void setAccelerationPriority(float priority) {
		image.setAccelerationPriority(priority);
	}
	public float getAccelerationPriority() {
		return image.getAccelerationPriority();
	}
	public int getType() {
		return image.getType();
	}
	public ColorModel getColorModel() {
		return image.getColorModel();
	}
	public WritableRaster getRaster() {
		return image.getRaster();
	}
	public WritableRaster getAlphaRaster() {
		return image.getAlphaRaster();
	}
	public int getRGB(int x, int y) {
		return image.getRGB(x, y);
	}
	public int[] getRGB(int startX, int startY, int w, int h, int[] rgbArray,
			int offset, int scansize) {
		return image.getRGB(startX, startY, w, h, rgbArray, offset, scansize);
	}
	public void setRGB(int x, int y, int rgb) {
		image.setRGB(x, y, rgb);
	}
	public void setRGB(int startX, int startY, int w, int h, int[] rgbArray,
			int offset, int scansize) {
		image.setRGB(startX, startY, w, h, rgbArray, offset, scansize);
	}
	public int getWidth() {
		return image.getWidth();
	}
	public int getHeight() {
		return image.getHeight();
	}
	public int getWidth(ImageObserver observer) {
		return image.getWidth(observer);
	}
	public int getHeight(ImageObserver observer) {
		return image.getHeight(observer);
	}
	public ImageProducer getSource() {
		return image.getSource();
	}
	public Object getProperty(String name, ImageObserver observer) {
		return image.getProperty(name, observer);
	}
	public Object getProperty(String name) {
		return image.getProperty(name);
	}
	public Graphics getGraphics() {
		return image.getGraphics();
	}
	public Graphics2D createGraphics() {
		return image.createGraphics();
	}
	public BufferedImage getSubimage(int x, int y, int w, int h) {
		return image.getSubimage(x, y, w, h);
	}
	public boolean isAlphaPremultiplied() {
		return image.isAlphaPremultiplied();
	}
	public void coerceData(boolean isAlphaPremultiplied) {
		image.coerceData(isAlphaPremultiplied);
	}
	public String toString() {
		return image.toString();
	}
	public Vector<RenderedImage> getSources() {
		return image.getSources();
	}
	public String[] getPropertyNames() {
		return image.getPropertyNames();
	}
	public int getMinX() {
		return image.getMinX();
	}
	public int getMinY() {
		return image.getMinY();
	}
	public SampleModel getSampleModel() {
		return image.getSampleModel();
	}
	public int getNumXTiles() {
		return image.getNumXTiles();
	}
	public int getNumYTiles() {
		return image.getNumYTiles();
	}
	public int getMinTileX() {
		return image.getMinTileX();
	}
	public int getMinTileY() {
		return image.getMinTileY();
	}
	public int getTileWidth() {
		return image.getTileWidth();
	}
	public int getTileHeight() {
		return image.getTileHeight();
	}
	public int getTileGridXOffset() {
		return image.getTileGridXOffset();
	}
	public int getTileGridYOffset() {
		return image.getTileGridYOffset();
	}
	public Raster getTile(int tileX, int tileY) {
		return image.getTile(tileX, tileY);
	}
	public Raster getData() {
		return image.getData();
	}
	public Raster getData(Rectangle rect) {
		return image.getData(rect);
	}
	public WritableRaster copyData(WritableRaster outRaster) {
		return image.copyData(outRaster);
	}
	public void setData(Raster r) {
		image.setData(r);
	}
	public void addTileObserver(TileObserver to) {
		image.addTileObserver(to);
	}
	public void removeTileObserver(TileObserver to) {
		image.removeTileObserver(to);
	}
	public boolean isTileWritable(int tileX, int tileY) {
		return image.isTileWritable(tileX, tileY);
	}
	public Point[] getWritableTileIndices() {
		return image.getWritableTileIndices();
	}
	public boolean hasTileWriters() {
		return image.hasTileWriters();
	}
	public WritableRaster getWritableTile(int tileX, int tileY) {
		return image.getWritableTile(tileX, tileY);
	}
	public void releaseWritableTile(int tileX, int tileY) {
		image.releaseWritableTile(tileX, tileY);
	}
	public int getTransparency() {
		return image.getTransparency();
	}
	
}
